package com.diagbot.util;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import io.micrometer.core.instrument.util.StringUtils;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Description: xml解析工具
 * @author: dsYun
 * @time: 2024/7/11 14:23
 */
public class XmlUtil {

    private static String LESS_THAN_SYMBOL_REPLACE = "th@xyh";
    private static String START_SURROUND_SYMBOL = "th@ksxyh";
    private static String END_SURROUND_SYMBOL = "th@jsdyh";

    /**
     * getText()获取当前节点的文本内容，getStringValue()获取所有子节点拼接后的文本内容
     *
     * @param xml
     * @return
     */
    public static Map<String, String> getXmlToMap(String xml) {
        Map<String, String> retMap = Maps.newLinkedHashMap();
        try {
            Document doc = DocumentHelper.parseText(xmlErrorCorrection(xml));
            List<Element> emrTermElements = doc.getRootElement().elements();
            String key, value;
            for (Element emrTermElement : emrTermElements) {
                key = emrTermElement.attributeValue("title");
                value = emrTermElement.getStringValue();
                if (StringUtil.isBlank(value)) {
                    continue;
                }
                value = value.trim().replaceAll("&nbsp;", "").replace("\n", "");
                if (StringUtil.isNotBlank(key) && StringUtil.isNotBlank(value)) {
                    retMap.put(key, value);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        //还原value值
        Map<String, String> correctMap = correctMapOfXml(retMap);
        return specialHandle(correctMap);
    }

    /**
     * 纠正xml内容，将因为不规范而影响dom4j解析的部分先替换掉，解析完后再替换回来
     *
     * @param xml
     * @return
     */
    public static String xmlErrorCorrection(String xml) {
//        xml = xml.replaceAll("\\<\\?xml[\\S\\s][^<>]*\\?\\>", "").replace("</zlxml>","");
        int index = xml.indexOf("</revisioninfos>");
        xml = xml.substring(index + "</revisioninfos>".length()).replace("</zlxml>", "");
        StringBuffer sbf = new StringBuffer();
        Pattern pattern = Pattern.compile("\\<.[^<>]*\\>");
        Matcher matcher = pattern.matcher(xml);
        int start = 0, end = 0;
        String xmlCopy = xml, elementName = null, elementNameReplace = null;
        while (matcher.find()) {
            elementName = matcher.group();
            start = matcher.start();
            if (end > 0 && start > 0 && end < start) {
                sbf.append(xml.substring(end, start).replaceAll("<", LESS_THAN_SYMBOL_REPLACE));
            }
            if (isCorrectXmlElement(elementName, xmlCopy)) {
                sbf.append(elementName);
            } else {
                elementNameReplace = START_SURROUND_SYMBOL + EnDecodeUtil.encode(elementName) + END_SURROUND_SYMBOL;
                xmlCopy = xmlCopy.replace(elementName, elementNameReplace);
                sbf.append(elementNameReplace);
            }
            end = matcher.end();
        }
        return sbf.toString();
    }

    /**
     * 判断某个xml标签，在当前xml文档中，是否是正常的xml标签
     *
     * @param elementName 标签
     * @param xml         当前xml文档
     * @return
     */
    public static boolean isCorrectXmlElement(String elementName, String xml) {
        //过滤类型：<>、< abc>
        if (elementName.length() == 2 || elementName.indexOf(" ") == 1) {
            return false;
        }
        //过滤类型：<abc/ >
        String elementNameCopy = elementName.substring(1, elementName.length() - 1).trim();
        if (elementNameCopy.endsWith("/") && elementName.lastIndexOf(" ") == elementName.length() - 2) {
            return false;
        }
        //过滤类型：有<abc>，而没有</abc>
        if (!elementNameCopy.endsWith("/")) {
            if (elementNameCopy.startsWith("/")) {
                Pattern pattern = Pattern.compile("\\<" + elementNameCopy.substring(1) + ".[^<>]*\\>");
                Matcher matcher = pattern.matcher(xml);
                if (xml.indexOf("<" + elementNameCopy.substring(1) + ">") == -1 && !matcher.find()) {
                    return false;
                }
            } else {
                int firstBlankIndex = elementNameCopy.indexOf(" ");
                firstBlankIndex = firstBlankIndex == -1 ? elementNameCopy.length() : firstBlankIndex;
                elementNameCopy = elementNameCopy.substring(0, firstBlankIndex);
                if (xml.indexOf("</" + elementNameCopy + ">") == -1) {
                    return false;
                }
            }
        }
        return true;
    }

    /**
     * 对由xml解析得到的map进行遍历，将value值中被替换的部分还原回来，
     *
     * @param sourceMap
     * @return
     */
    public static Map<String, String> correctMapOfXml(Map<String, String> sourceMap) {
        Pattern pattern = Pattern.compile(START_SURROUND_SYMBOL + ".*" + END_SURROUND_SYMBOL);
        String value = null, valueCopy = null, matMsg = null;
        Matcher matcher = null;
        for (String key : sourceMap.keySet()) {
            value = sourceMap.get(key);
            valueCopy = value;
            valueCopy = valueCopy.replaceAll(LESS_THAN_SYMBOL_REPLACE, "<");
            matcher = pattern.matcher(value);
            while (matcher.find()) {
                matMsg = matcher.group();
                valueCopy = valueCopy.replace(matMsg,
                        EnDecodeUtil.decode(
                                matMsg.replace(START_SURROUND_SYMBOL, "").replace(END_SURROUND_SYMBOL, "")
                        )
                );
            }
            sourceMap.put(key, valueCopy);
        }
        return sourceMap;
    }

    /**
     * 针对字段进行特殊处理，
     *
     * @param retMap
     * @return
     */
    public static Map<String, String> specialHandle(Map<String, String> retMap) {
        if (retMap.containsKey("姓吠")) {
            retMap.put("姓名", retMap.get("姓吠"));
        }
        if (retMap.containsKey("性别")) {
            retMap.put("性别", retMap.get("性别").substring(retMap.get("性别").length() - 1));
        }
        //过去史分解
//        List<String> pastTitles = Lists.newArrayList("既往史", "家族史", "过敏史");
//        structureMap(pastTitles, "既往史" + retMap.get("过去史"), retMap);

        //查体分解
        List<String> vitalTitles = Lists.newArrayList("查体", "诊断", "建议", "科室", "医师签名", "就诊时间", "处置");
        structureMap(vitalTitles, "查体" + retMap.get("体检").replace("建议 ", "建议"), retMap);

        if (retMap.containsKey("建议")) {
            String drugStr = retMap.get("建议").replace("--------------------- ", "");
            if (drugStr.contains(" ")) {
                retMap.put("药品", drugStr.substring(0, drugStr.indexOf(" ")));
            } else {
                retMap.put("医嘱", drugStr);
            }
        }
        return retMap;
    }

    public static void structureMap(List<String> titles, String text, Map<String, String> structureMap) {
//        text = text.replace(" ","");
        List<String> sortTitles = sortTitles(titles, text);
        cutByTitles(text, sortTitles, 0, structureMap);
    }

    /**
     * 将title根据在文本中的位置排序
     *
     * @param titles
     * @param content
     * @return
     */
    public static List<String> sortTitles(List<String> titles, String content) {
        Map<Integer, String> titleIndex = new TreeMap<>();
        int index, index_1, index_2;
        for (String title : titles) {
            index_1 = content.indexOf(title + "：");
            index_2 = content.indexOf(title + ":");
            index = Math.max(index_1, index_2);
            if (index != -1) {
                titleIndex.put(index, title);
                StringBuffer sb = new StringBuffer(title.length());
                for (int i = 0; i < title.length(); i++) {
                    sb.append('*');
                }
                content = content.substring(0, index) + sb.toString() + content.substring(index + title.length() + 1);
                //                content = content.substring(0, index) + content.substring(index + title.length() + 1);
            }
        }
        titles = Lists.newArrayList(titleIndex.values());
        return titles;
    }

    /**
     * 根据文书各标题截取相应文本，存入structmap中
     *
     * @param line         原始文本
     * @param titles       文书各标题
     * @param depth        递归深度，也就是titles取值时的下标值
     * @param structureMap 存储结构化数据
     */
    public static void cutByTitles(String line, List<String> titles, int depth, Map<String, String> structureMap) {
        if (depth > titles.size() || titles.size() == 0) {
            return;
        }
        String beforeTitle = null, title = null, newTitle = null, value = null;
        beforeTitle = removeBlank(titles.get(Math.max(depth - 1, 0)));
        title = titles.get(Math.min(depth, titles.size() - 1));
        if (depth == titles.size()) {
            /*if (line.contains("\n")) {
                line = line.split("\n")[0];
            }
            */
            value = line.replace("\n", "");
            if (StringUtil.isBlank(structureMap.get(beforeTitle))) {
                structureMap.put(beforeTitle, trim(value));
            }
            return;
        }
        if (line.contains(title + "：") || line.contains(title + ":")) {
            if (line.contains(title + "：")) {
                newTitle = title + "：";
            } else {
                newTitle = title + ":";
            }
            if (depth > 0) {
                value = line.substring(0, line.indexOf(newTitle));
                if (StringUtil.isBlank(structureMap.get(beforeTitle))) {
                    structureMap.put(beforeTitle, trim(value).replace("\n", ""));
                }
            }
            line = line.substring(line.indexOf(newTitle) + newTitle.length());
            depth++;
        } else {
            titles.remove(depth);
        }
        cutByTitles(line, titles, depth, structureMap);
    }

    /**
     * 清除字符串中的空白
     *
     * @param str
     * @return
     */
    public static String removeBlank(String str) {
        if (StringUtils.isBlank(str)) {
            return str;
        }
        return str.replaceAll("[\\s\\p{Zs}]", "").replaceAll("　", "");
    }

    /**
     * 去除字符串两边空白 包含一些特殊空格
     *
     * @param msg
     * @return
     */
    public static String trim(String msg) {
        if (StringUtils.isNotBlank(msg)) {
            return org.springframework.util.StringUtils.trimWhitespace(msg).trim();
        }
        return msg;
    }

}